#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import logging.config
import sys
reload(sys)
sys.setdefaultencoding('utf-8')

import collections
import time
import urllib2
import urllib
import socket
import json
from xml.etree import ElementTree
from clfcfg import CLFCfg
from ConfigParser import ConfigParser

class LDAPQueryHandler(object):

    def __init__(self):
        #cache struct:
        # 'username': {
        #   'groups': ['group1', 'group2', ...],
        #   'expire': xxx
        # }
        CLFCfg.init()
        self.__user_group_cache = collections.defaultdict()
        self.__expire = CLFCfg.config.getint('LDAP', 'expire')
        if self.__expire <= 0:
            self.__expire = 7200

    def check_cache(self, user, group):
        ts = int(time.time())

        if self.__user_group_cache.has_key(user):
            user_info = self.__user_group_cache[user]
            if ts < user_info['expire']:
                return (True, group in user_info['groups'])
            else:
                self.__user_group_cache.pop(user)
        return (False, False)

    def update_cache(self, user, groups, exp_time = 0):
        if exp_time == 0:
            exp_time = self.__expire
        expire_time = int(time.time()) + exp_time
        self.__user_group_cache[user] = {'groups': groups, 'expire': expire_time}

    def is_user_in_group(self, user, group):
        pass

    def get_members(self, group):
        pass

class IWSSLDAPQueryHandler(LDAPQueryHandler):
    def __init__(self):
        LDAPQueryHandler.__init__(self)
        cp = ConfigParser()
        cp.read('/etc/iscan/intscan.ini')
        proto = cp.get('http', 'iscan_web_protocol')
        admin_port = cp.get('http', 'iscan_web_server')
        if admin_port.find(':') > 0:
            admin_port = admin_port.split(':')[-1]
        self.server_address = '%s://127.0.0.1:%s' % (proto, admin_port)
        logging.debug('IWSVA web console: %s', self.server_address)

    def is_user_in_group(self, user, group):

        #check cache first
        (is_cache, result) = self.check_cache(user, group)
        if is_cache:
            return result

        try: 
           socket.inet_aton(user)
           #Skip IP address
           return False        
        except socket.error:
           pass

        #query LDAP for user
        url_values = urllib.urlencode({'user': user})
        try:
            response = urllib2.urlopen(self.server_address + '/rest/user_groups?' + url_values)
        except Exception as e:
            logging.debug('is_user_in_group failed for user [%r]: message: %r', user, e)
            #add to cache 30s to avoid query consistently
            self.update_cache(user, [], 30)
            return False

        result = response.read()

        root = ElementTree.fromstring(result)
        group_nodes = root.findall('name')
        groups = []
        for g in group_nodes:
            groups.append(g.text)

        #put to cache
        self.update_cache(user, groups)

        return group in groups

    def get_members(self, group):
        try:
            response = urllib2.urlopen(self.server_address + '/rest/groupuserlist?isdn=0&groups=' + urllib.quote_plus(group))
            
            result = response.read()
            group1 = group.replace('&amp;', '&').replace('&lt;','<').replace('&gt;','>').replace('&quot;','"').replace('&apos;','\'')
            members = json.loads(result)[group1]
        except Exception as e:
            logging.error('get_members failed for [%r]: %r', group, e)
            return []

        return members

class DeepEdgeLDAPQueryHandler(LDAPQueryHandler):
    def __init__(self):
        LDAPQueryHandler.__init__(self)
        self.deInterface = __import__("interface.UserId_Impl.UserGroup")

    def is_user_in_group(self, user, group):
        #check cache first
        (is_cache, result) = self.check_cache(user, group)
        if is_cache:
            return result
        groupList = []
        try:
            UserInfoList = self.deInterface.UserId_Impl.UserGroup.queryUser(user.encode('utf-8'))
            if len(UserInfoList) > 0:
                userid = UserInfoList[0]["userName"]
            
                groupInfo = self.deInterface.UserId_Impl.UserGroup.queryUserGroup(userid.encode("ascii"))
                for group in groupInfo:
                    groupList.append(group["commonName"])
        except:
            pass

        self.update_cache(user, groupList)

        return group in groupList


class SkynetLDAPQueryHandler():
    def __init__(self):
        config = ConfigParser()
        config.read('/etc/skynet/skynet-env.ini')
        self.userid_query_url = config.get('UserIdSvcRole', 'USERID_SVC_URL') + 'list-all-users-in-group'
        self.group_query_url = config.get('UserIdSvcRole', 'USERID_SVC_URL') + 'account-search'
        # {'groupid': {'userid1': ts} ... }
        self.group_user_cache = {}

    def is_user_in_group(self, user, group):
        logging.debug('enter query skynet user group: %r', user, group)
        return False
        #

def CreateLDAPQueryHandler():
    if CLFCfg.LDAP_enabled and CLFCfg.LDAP_mode == 'IWSVA':
        return IWSSLDAPQueryHandler()
    elif CLFCfg.LDAP_enabled and CLFCfg.LDAP_mode == 'SKYNET':
        return SkynetLDAPQueryHandler()
    elif CLFCfg.LDAP_enabled and CLFCfg.LDAP_mode == 'DEEP_EDGE':
        return DeepEdgeLDAPQueryHandler()
    else:
        return None

if __name__ == '__main__':
    pass

